@@ -3,7 +3,7 @@ from pathlib import Path
 
 import pytest
 from ape.exceptions import ContractLogicError
-from ape.types import CallTreeNode
+from ape.types import CallTreeNode, TraceFrame
 from evm_trace import CallType
 from hexbytes import HexBytes
 
@@ -100,6 +100,16 @@ def test_unlock_account(connected_provider):
     assert TEST_WALLET_ADDRESS in connected_provider.unlocked_accounts
 
 
+def test_get_transaction_trace(connected_provider, contract_instance, owner):
+    receipt = contract_instance.setNumber(10, sender=owner)
+
+    # Indirectly calls `connected_provider.get_transaction_trace()`
+    frame_data = list(receipt.trace)
+    assert frame_data
+    for frame in frame_data:
+        assert isinstance(frame, TraceFrame)
+
+
 def test_get_call_tree(connected_provider, sender, receiver):
     transfer = sender.transfer(receiver, 1)
     call_tree = connected_provider.get_call_tree(transfer.txn_hash)
